import java.util.*;
import java.awt.*;
import java.applet.*;
import java.awt.event.*;

public class GraphPanel extends Component {
    private Dimension size = new Dimension();
    private int x[], xmax, xmin;
    private int y[], ymax, ymin;
    private GraphApplet caller;

    /** ν */
    final static Color DEFAULT_NODE_COLOR = Color.green;
    /** ޤν */
    final static Color DEFAULT_EDGE_COLOR = Color.orange;
    /** 礭 */
    final static int RADIUS = 3;

    /** ο */
    private Color nodeColor = DEFAULT_NODE_COLOR;
    /** ޤο */
    private Color edgeColor = DEFAULT_EDGE_COLOR;
    /** ޤطʿ(null Ǥʤ XOR 褹) */
    private Color edgeBgColor = null;

    /** ޥĺ(ĺΩĤ褦褹) */
    public boolean mouseHandle = false;
    /** ޥĺǤ */
    public boolean capturing = false;
    /** ǤĺΥǥå */
    public int index = 0;
    //public boolean clicked = false;

    public GraphPanel( GraphApplet caller ) {
        this.caller = caller;
        Mouse m = new Mouse();
        addMouseListener(m);
    }

    public void setColors( Color node, Color edge, Color edgeBg ) {
        nodeColor = (node == null) ? DEFAULT_NODE_COLOR : node;
        edgeColor = (edge == null) ? DEFAULT_EDGE_COLOR : edge;
        edgeBgColor = edgeBg;
    }

    public void setNodes(int[] x, int[] y) {
        this.x = x;
        this.y = y;
        if (x.length != y.length) {
            throw new IllegalArgumentException("x.length != y.length");
        }
        xmax = ymax = Integer.MIN_VALUE;
        xmin = ymin = Integer.MAX_VALUE;
        for (int i = 0; i < x.length; i++) {
            if (xmax < x[i]) xmax = x[i];
            if (xmin > x[i]) xmin = x[i];
            if (ymax < y[i]) ymax = y[i];
            if (ymin > y[i]) ymin = y[i];
        }
    }

    private int screenX(int x) {
	return 20 + (size.width - 40) * (x - xmin) / (xmax - xmin);
    }

    private int screenY(int y) {
	return 20 + (size.height - 40) * (y - ymax) / (ymin - ymax);
    }

    public void drawEdge( int i, int j ) {
        Graphics g = getGraphics();
        if ( g == null ) return; // ʤϤ
	g.setColor( edgeColor );
	if ( edgeBgColor != null ) g.setXORMode( edgeBgColor );
	drawEdge( i, j, g );
    }

    public synchronized void drawEdge( int i, int j, Graphics g ) {
        if ( i > j ) {  int swap = i;  i = j;  j = swap;  }
	g.drawLine( screenX(x[i]), screenY(y[i]),
                    screenX(x[j]), screenY(y[j]) );
    }

    private void drawNodeCircle() {
        Graphics g = getGraphics();
        if ( g == null ) return; // ʤϤ
        drawNodeCircle( g );
    }

    private void drawNodeCircle( Graphics g ) {
	g.setColor( edgeColor );
	g.setXORMode( getBackground() );
        g.fillArc( screenX(x[index])-RADIUS*2,
                   screenY(y[index])-RADIUS*2,
                   RADIUS*4+1, RADIUS*4+1, 0, 360 );
    }

    public synchronized void paint( Graphics g ) {
	getSize(size);
	g.setPaintMode();
	g.setColor( getBackground() );
	g.fillRect( 0, 0, size.width, size.height );
        if ( x == null ) return;

	g.setColor( edgeColor );
	if ( edgeBgColor != null ) g.setXORMode( edgeBgColor );
        caller.drawAllEdges( g );

	g.setColor( nodeColor );
	for ( int i=0; i<x.length; i++ ) {
	    g.fillArc( screenX(x[i])-RADIUS,
                       screenY(y[i])-RADIUS,
                       RADIUS*2+1, RADIUS*2+1, 0, 360 );
	}

        if ( mouseHandle ) drawNodeCircle( g );
    }

    private int getNearestIndex( int mouseX, int mouseY ) {
        int index = -1;
        int dst2 = Integer.MAX_VALUE;
        for (int i=0; i<x.length; i++) {
            int dx = mouseX - screenX(x[i]);
            int dy = mouseY - screenY(y[i]);
            int tmp = dx*dx + dy*dy;
            if (dst2 > tmp) {
                dst2 = tmp;
                index = i;
            }
        }
        return index;
    }

    public void setMouseHandle( boolean enable ) {
        if ( mouseHandle ^ enable ) drawNodeCircle();
        mouseHandle = capturing = enable;
    }

    private class Mouse extends MouseAdapter {
        public void mouseClicked(MouseEvent e) {
            if (!capturing) return;
            int tmp = getNearestIndex(e.getX(), e.getY());
            if (index != tmp) {
                drawNodeCircle();
                index = tmp;
                drawNodeCircle();
            }
        }
    }
}
